# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Union, cast
from datetime import datetime

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.d1.database import time_travel_restore_params, time_travel_get_bookmark_params
from ....types.d1.database.time_travel_restore_response import TimeTravelRestoreResponse
from ....types.d1.database.time_travel_get_bookmark_response import TimeTravelGetBookmarkResponse

__all__ = ["TimeTravelResource", "AsyncTimeTravelResource"]


class TimeTravelResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TimeTravelResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TimeTravelResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TimeTravelResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TimeTravelResourceWithStreamingResponse(self)

    def get_bookmark(
        self,
        database_id: str,
        *,
        account_id: str,
        timestamp: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TimeTravelGetBookmarkResponse:
        """
        Retrieves the current bookmark, or the nearest bookmark at or before a provided
        timestamp. Bookmarks can be used with the restore endpoint to revert the
        database to a previous point in time.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          timestamp: An optional ISO 8601 timestamp. If provided, returns the nearest available
              bookmark at or before this timestamp. If omitted, returns the current bookmark.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not database_id:
            raise ValueError(f"Expected a non-empty value for `database_id` but received {database_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/d1/database/{database_id}/time_travel/bookmark",
                account_id=account_id,
                database_id=database_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"timestamp": timestamp}, time_travel_get_bookmark_params.TimeTravelGetBookmarkParams
                ),
                post_parser=ResultWrapper[TimeTravelGetBookmarkResponse]._unwrapper,
            ),
            cast_to=cast(Type[TimeTravelGetBookmarkResponse], ResultWrapper[TimeTravelGetBookmarkResponse]),
        )

    def restore(
        self,
        database_id: str,
        *,
        account_id: str,
        bookmark: str | Omit = omit,
        timestamp: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TimeTravelRestoreResponse:
        """
        Restores a D1 database to a previous point in time either via a bookmark or a
        timestamp.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          bookmark: A bookmark to restore the database to. Required if `timestamp` is not provided.

          timestamp: An ISO 8601 timestamp to restore the database to. Required if `bookmark` is not
              provided.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not database_id:
            raise ValueError(f"Expected a non-empty value for `database_id` but received {database_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/d1/database/{database_id}/time_travel/restore",
                account_id=account_id,
                database_id=database_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "bookmark": bookmark,
                        "timestamp": timestamp,
                    },
                    time_travel_restore_params.TimeTravelRestoreParams,
                ),
                post_parser=ResultWrapper[TimeTravelRestoreResponse]._unwrapper,
            ),
            cast_to=cast(Type[TimeTravelRestoreResponse], ResultWrapper[TimeTravelRestoreResponse]),
        )


class AsyncTimeTravelResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTimeTravelResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTimeTravelResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTimeTravelResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTimeTravelResourceWithStreamingResponse(self)

    async def get_bookmark(
        self,
        database_id: str,
        *,
        account_id: str,
        timestamp: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TimeTravelGetBookmarkResponse:
        """
        Retrieves the current bookmark, or the nearest bookmark at or before a provided
        timestamp. Bookmarks can be used with the restore endpoint to revert the
        database to a previous point in time.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          timestamp: An optional ISO 8601 timestamp. If provided, returns the nearest available
              bookmark at or before this timestamp. If omitted, returns the current bookmark.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not database_id:
            raise ValueError(f"Expected a non-empty value for `database_id` but received {database_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/d1/database/{database_id}/time_travel/bookmark",
                account_id=account_id,
                database_id=database_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"timestamp": timestamp}, time_travel_get_bookmark_params.TimeTravelGetBookmarkParams
                ),
                post_parser=ResultWrapper[TimeTravelGetBookmarkResponse]._unwrapper,
            ),
            cast_to=cast(Type[TimeTravelGetBookmarkResponse], ResultWrapper[TimeTravelGetBookmarkResponse]),
        )

    async def restore(
        self,
        database_id: str,
        *,
        account_id: str,
        bookmark: str | Omit = omit,
        timestamp: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TimeTravelRestoreResponse:
        """
        Restores a D1 database to a previous point in time either via a bookmark or a
        timestamp.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          bookmark: A bookmark to restore the database to. Required if `timestamp` is not provided.

          timestamp: An ISO 8601 timestamp to restore the database to. Required if `bookmark` is not
              provided.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not database_id:
            raise ValueError(f"Expected a non-empty value for `database_id` but received {database_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/d1/database/{database_id}/time_travel/restore",
                account_id=account_id,
                database_id=database_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "bookmark": bookmark,
                        "timestamp": timestamp,
                    },
                    time_travel_restore_params.TimeTravelRestoreParams,
                ),
                post_parser=ResultWrapper[TimeTravelRestoreResponse]._unwrapper,
            ),
            cast_to=cast(Type[TimeTravelRestoreResponse], ResultWrapper[TimeTravelRestoreResponse]),
        )


class TimeTravelResourceWithRawResponse:
    def __init__(self, time_travel: TimeTravelResource) -> None:
        self._time_travel = time_travel

        self.get_bookmark = to_raw_response_wrapper(
            time_travel.get_bookmark,
        )
        self.restore = to_raw_response_wrapper(
            time_travel.restore,
        )


class AsyncTimeTravelResourceWithRawResponse:
    def __init__(self, time_travel: AsyncTimeTravelResource) -> None:
        self._time_travel = time_travel

        self.get_bookmark = async_to_raw_response_wrapper(
            time_travel.get_bookmark,
        )
        self.restore = async_to_raw_response_wrapper(
            time_travel.restore,
        )


class TimeTravelResourceWithStreamingResponse:
    def __init__(self, time_travel: TimeTravelResource) -> None:
        self._time_travel = time_travel

        self.get_bookmark = to_streamed_response_wrapper(
            time_travel.get_bookmark,
        )
        self.restore = to_streamed_response_wrapper(
            time_travel.restore,
        )


class AsyncTimeTravelResourceWithStreamingResponse:
    def __init__(self, time_travel: AsyncTimeTravelResource) -> None:
        self._time_travel = time_travel

        self.get_bookmark = async_to_streamed_response_wrapper(
            time_travel.get_bookmark,
        )
        self.restore = async_to_streamed_response_wrapper(
            time_travel.restore,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .network_path import (
    NetworkPathResource,
    AsyncNetworkPathResource,
    NetworkPathResourceWithRawResponse,
    AsyncNetworkPathResourceWithRawResponse,
    NetworkPathResourceWithStreamingResponse,
    AsyncNetworkPathResourceWithStreamingResponse,
)
from .traceroute_test_results import (
    TracerouteTestResultsResource,
    AsyncTracerouteTestResultsResource,
    TracerouteTestResultsResourceWithRawResponse,
    AsyncTracerouteTestResultsResourceWithRawResponse,
    TracerouteTestResultsResourceWithStreamingResponse,
    AsyncTracerouteTestResultsResourceWithStreamingResponse,
)

__all__ = [
    "NetworkPathResource",
    "AsyncNetworkPathResource",
    "NetworkPathResourceWithRawResponse",
    "AsyncNetworkPathResourceWithRawResponse",
    "NetworkPathResourceWithStreamingResponse",
    "AsyncNetworkPathResourceWithStreamingResponse",
    "TracerouteTestResultsResource",
    "AsyncTracerouteTestResultsResource",
    "TracerouteTestResultsResourceWithRawResponse",
    "AsyncTracerouteTestResultsResourceWithRawResponse",
    "TracerouteTestResultsResourceWithStreamingResponse",
    "AsyncTracerouteTestResultsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = [
    "LifecycleGetResponse",
    "Rule",
    "RuleConditions",
    "RuleAbortMultipartUploadsTransition",
    "RuleAbortMultipartUploadsTransitionCondition",
    "RuleDeleteObjectsTransition",
    "RuleDeleteObjectsTransitionCondition",
    "RuleDeleteObjectsTransitionConditionR2LifecycleAgeCondition",
    "RuleDeleteObjectsTransitionConditionR2LifecycleDateCondition",
    "RuleStorageClassTransition",
    "RuleStorageClassTransitionCondition",
    "RuleStorageClassTransitionConditionR2LifecycleAgeCondition",
    "RuleStorageClassTransitionConditionR2LifecycleDateCondition",
]


class RuleConditions(BaseModel):
    """Conditions that apply to all transitions of this rule."""

    prefix: str
    """
    Transitions will only apply to objects/uploads in the bucket that start with the
    given prefix, an empty prefix can be provided to scope rule to all
    objects/uploads.
    """


class RuleAbortMultipartUploadsTransitionCondition(BaseModel):
    """
    Condition for lifecycle transitions to apply after an object reaches an age in seconds.
    """

    max_age: int = FieldInfo(alias="maxAge")

    type: Literal["Age"]


class RuleAbortMultipartUploadsTransition(BaseModel):
    """Transition to abort ongoing multipart uploads."""

    condition: Optional[RuleAbortMultipartUploadsTransitionCondition] = None
    """
    Condition for lifecycle transitions to apply after an object reaches an age in
    seconds.
    """


class RuleDeleteObjectsTransitionConditionR2LifecycleAgeCondition(BaseModel):
    """
    Condition for lifecycle transitions to apply after an object reaches an age in seconds.
    """

    max_age: int = FieldInfo(alias="maxAge")

    type: Literal["Age"]


class RuleDeleteObjectsTransitionConditionR2LifecycleDateCondition(BaseModel):
    """Condition for lifecycle transitions to apply on a specific date."""

    date: datetime

    type: Literal["Date"]


RuleDeleteObjectsTransitionCondition: TypeAlias = Union[
    RuleDeleteObjectsTransitionConditionR2LifecycleAgeCondition,
    RuleDeleteObjectsTransitionConditionR2LifecycleDateCondition,
]


class RuleDeleteObjectsTransition(BaseModel):
    """Transition to delete objects."""

    condition: Optional[RuleDeleteObjectsTransitionCondition] = None
    """
    Condition for lifecycle transitions to apply after an object reaches an age in
    seconds.
    """


class RuleStorageClassTransitionConditionR2LifecycleAgeCondition(BaseModel):
    """
    Condition for lifecycle transitions to apply after an object reaches an age in seconds.
    """

    max_age: int = FieldInfo(alias="maxAge")

    type: Literal["Age"]


class RuleStorageClassTransitionConditionR2LifecycleDateCondition(BaseModel):
    """Condition for lifecycle transitions to apply on a specific date."""

    date: datetime

    type: Literal["Date"]


RuleStorageClassTransitionCondition: TypeAlias = Union[
    RuleStorageClassTransitionConditionR2LifecycleAgeCondition,
    RuleStorageClassTransitionConditionR2LifecycleDateCondition,
]


class RuleStorageClassTransition(BaseModel):
    condition: RuleStorageClassTransitionCondition
    """
    Condition for lifecycle transitions to apply after an object reaches an age in
    seconds.
    """

    storage_class: Literal["InfrequentAccess"] = FieldInfo(alias="storageClass")


class Rule(BaseModel):
    id: str
    """Unique identifier for this rule."""

    conditions: RuleConditions
    """Conditions that apply to all transitions of this rule."""

    enabled: bool
    """Whether or not this rule is in effect."""

    abort_multipart_uploads_transition: Optional[RuleAbortMultipartUploadsTransition] = FieldInfo(
        alias="abortMultipartUploadsTransition", default=None
    )
    """Transition to abort ongoing multipart uploads."""

    delete_objects_transition: Optional[RuleDeleteObjectsTransition] = FieldInfo(
        alias="deleteObjectsTransition", default=None
    )
    """Transition to delete objects."""

    storage_class_transitions: Optional[List[RuleStorageClassTransition]] = FieldInfo(
        alias="storageClassTransitions", default=None
    )
    """Transitions to change the storage class of objects."""


class LifecycleGetResponse(BaseModel):
    rules: Optional[List[Rule]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, TypeAlias

from ...._models import BaseModel
from .file_input import FileInput
from .intune_input import IntuneInput
from .kolide_input import KolideInput
from .tanium_input import TaniumInput
from .firewall_input import FirewallInput
from .os_version_input import OSVersionInput
from .crowdstrike_input import CrowdstrikeInput
from .sentinelone_input import SentineloneInput
from .domain_joined_input import DomainJoinedInput
from .workspace_one_input import WorkspaceOneInput
from .disk_encryption_input import DiskEncryptionInput
from .sentinelone_s2s_input import SentineloneS2sInput
from .unique_client_id_input import UniqueClientIDInput
from .client_certificate_input import ClientCertificateInput

__all__ = [
    "DeviceInput",
    "TeamsDevicesCarbonblackInputRequest",
    "TeamsDevicesAccessSerialNumberListInputRequest",
    "TeamsDevicesApplicationInputRequest",
    "TeamsDevicesClientCertificateV2InputRequest",
    "TeamsDevicesClientCertificateV2InputRequestLocations",
    "TeamsDevicesAntivirusInputRequest",
    "TeamsDevicesCustomS2sInputRequest",
]


class TeamsDevicesCarbonblackInputRequest(BaseModel):
    operating_system: Literal["windows", "linux", "mac"]
    """Operating system."""

    path: str
    """File path."""

    sha256: Optional[str] = None
    """SHA-256."""

    thumbprint: Optional[str] = None
    """Signing certificate thumbprint."""


class TeamsDevicesAccessSerialNumberListInputRequest(BaseModel):
    id: str
    """UUID of Access List."""


class TeamsDevicesApplicationInputRequest(BaseModel):
    operating_system: Literal["windows", "linux", "mac"]
    """Operating system."""

    path: str
    """Path for the application."""

    sha256: Optional[str] = None
    """SHA-256."""

    thumbprint: Optional[str] = None
    """Signing certificate thumbprint."""


class TeamsDevicesClientCertificateV2InputRequestLocations(BaseModel):
    paths: Optional[List[str]] = None
    """List of paths to check for client certificate on linux."""

    trust_stores: Optional[List[Literal["system", "user"]]] = None
    """List of trust stores to check for client certificate."""


class TeamsDevicesClientCertificateV2InputRequest(BaseModel):
    certificate_id: str
    """UUID of Cloudflare managed certificate."""

    check_private_key: bool
    """Confirm the certificate was not imported from another device.

    We recommend keeping this enabled unless the certificate was deployed without a
    private key.
    """

    operating_system: Literal["windows", "linux", "mac"]
    """Operating system."""

    cn: Optional[str] = None
    """Certificate Common Name.

    This may include one or more variables in the ${ } notation. Only
    ${serial_number} and ${hostname} are valid variables.
    """

    extended_key_usage: Optional[List[Literal["clientAuth", "emailProtection"]]] = None
    """
    List of values indicating purposes for which the certificate public key can be
    used.
    """

    locations: Optional[TeamsDevicesClientCertificateV2InputRequestLocations] = None

    subject_alternative_names: Optional[List[str]] = None
    """List of certificate Subject Alternative Names."""


class TeamsDevicesAntivirusInputRequest(BaseModel):
    update_window_days: Optional[float] = None
    """Number of days that the antivirus should be updated within."""


class TeamsDevicesCustomS2sInputRequest(BaseModel):
    connection_id: str
    """Posture Integration ID."""

    operator: Literal["<", "<=", ">", ">=", "=="]
    """Operator."""

    score: float
    """
    A value between 0-100 assigned to devices set by the 3rd party posture provider.
    """


DeviceInput: TypeAlias = Union[
    FileInput,
    UniqueClientIDInput,
    DomainJoinedInput,
    OSVersionInput,
    FirewallInput,
    SentineloneInput,
    TeamsDevicesCarbonblackInputRequest,
    TeamsDevicesAccessSerialNumberListInputRequest,
    DiskEncryptionInput,
    TeamsDevicesApplicationInputRequest,
    ClientCertificateInput,
    TeamsDevicesClientCertificateV2InputRequest,
    TeamsDevicesAntivirusInputRequest,
    WorkspaceOneInput,
    CrowdstrikeInput,
    IntuneInput,
    KolideInput,
    TaniumInput,
    SentineloneS2sInput,
    TeamsDevicesCustomS2sInputRequest,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...types.ssl import RequestValidity
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.ssl.request_validity import RequestValidity
from ...types.origin_ca_certificates import origin_ca_certificate_list_params, origin_ca_certificate_create_params
from ...types.shared.certificate_request_type import CertificateRequestType
from ...types.origin_ca_certificates.origin_ca_certificate import OriginCACertificate
from ...types.origin_ca_certificates.origin_ca_certificate_delete_response import OriginCACertificateDeleteResponse

__all__ = ["OriginCACertificatesResource", "AsyncOriginCACertificatesResource"]


class OriginCACertificatesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> OriginCACertificatesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return OriginCACertificatesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OriginCACertificatesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return OriginCACertificatesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        csr: str,
        hostnames: SequenceNotStr[str],
        request_type: CertificateRequestType,
        requested_validity: RequestValidity | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OriginCACertificate]:
        """Create an Origin CA certificate.

        You can use an Origin CA Key as your User
        Service Key or an API token when calling this endpoint ([see above](#requests)).

        Args:
          csr: The Certificate Signing Request (CSR). Must be newline-encoded.

          hostnames: Array of hostnames or wildcard names bound to the certificate. Hostnames must be
              fully qualified domain names (FQDNs) belonging to zones on your account (e.g.,
              `example.com` or `sub.example.com`). Wildcards are supported only as a `*.`
              prefix for a single level (e.g., `*.example.com`). Double wildcards
              (`*.*.example.com`) and interior wildcards (`foo.*.example.com`) are not
              allowed. The wildcard suffix must be a multi-label domain (`*.example.com` is
              valid, but `*.com` is not). Unicode/IDN hostnames are accepted and automatically
              converted to punycode.

          request_type: Signature type desired on certificate ("origin-rsa" (rsa), "origin-ecc" (ecdsa),
              or "keyless-certificate" (for Keyless SSL servers).

          requested_validity: The number of days for which the certificate should be valid.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/certificates",
            body=maybe_transform(
                {
                    "csr": csr,
                    "hostnames": hostnames,
                    "request_type": request_type,
                    "requested_validity": requested_validity,
                },
                origin_ca_certificate_create_params.OriginCACertificateCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OriginCACertificate]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OriginCACertificate]], ResultWrapper[OriginCACertificate]),
        )

    def list(
        self,
        *,
        zone_id: str,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[OriginCACertificate]:
        """List all existing Origin CA certificates for a given zone.

        You can use an Origin
        CA Key as your User Service Key or an API token when calling this endpoint
        ([see above](#requests)).

        Args:
          zone_id: Identifier.

          limit: Limit to the number of records returned.

          offset: Offset the results

          page: Page number of paginated results.

          per_page: Number of records per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/certificates",
            page=SyncV4PagePaginationArray[OriginCACertificate],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "zone_id": zone_id,
                        "limit": limit,
                        "offset": offset,
                        "page": page,
                        "per_page": per_page,
                    },
                    origin_ca_certificate_list_params.OriginCACertificateListParams,
                ),
            ),
            model=OriginCACertificate,
        )

    def delete(
        self,
        certificate_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OriginCACertificateDeleteResponse]:
        """Revoke an existing Origin CA certificate by its serial number.

        You can use an
        Origin CA Key as your User Service Key or an API token when calling this
        endpoint ([see above](#requests)).

        Args:
          certificate_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not certificate_id:
            raise ValueError(f"Expected a non-empty value for `certificate_id` but received {certificate_id!r}")
        return self._delete(
            path_template("/certificates/{certificate_id}", certificate_id=certificate_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OriginCACertificateDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[OriginCACertificateDeleteResponse]], ResultWrapper[OriginCACertificateDeleteResponse]
            ),
        )

    def get(
        self,
        certificate_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OriginCACertificate]:
        """Get an existing Origin CA certificate by its serial number.

        You can use an
        Origin CA Key as your User Service Key or an API token when calling this
        endpoint ([see above](#requests)).

        Args:
          certificate_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not certificate_id:
            raise ValueError(f"Expected a non-empty value for `certificate_id` but received {certificate_id!r}")
        return self._get(
            path_template("/certificates/{certificate_id}", certificate_id=certificate_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OriginCACertificate]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OriginCACertificate]], ResultWrapper[OriginCACertificate]),
        )


class AsyncOriginCACertificatesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncOriginCACertificatesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOriginCACertificatesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOriginCACertificatesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncOriginCACertificatesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        csr: str,
        hostnames: SequenceNotStr[str],
        request_type: CertificateRequestType,
        requested_validity: RequestValidity | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OriginCACertificate]:
        """Create an Origin CA certificate.

        You can use an Origin CA Key as your User
        Service Key or an API token when calling this endpoint ([see above](#requests)).

        Args:
          csr: The Certificate Signing Request (CSR). Must be newline-encoded.

          hostnames: Array of hostnames or wildcard names bound to the certificate. Hostnames must be
              fully qualified domain names (FQDNs) belonging to zones on your account (e.g.,
              `example.com` or `sub.example.com`). Wildcards are supported only as a `*.`
              prefix for a single level (e.g., `*.example.com`). Double wildcards
              (`*.*.example.com`) and interior wildcards (`foo.*.example.com`) are not
              allowed. The wildcard suffix must be a multi-label domain (`*.example.com` is
              valid, but `*.com` is not). Unicode/IDN hostnames are accepted and automatically
              converted to punycode.

          request_type: Signature type desired on certificate ("origin-rsa" (rsa), "origin-ecc" (ecdsa),
              or "keyless-certificate" (for Keyless SSL servers).

          requested_validity: The number of days for which the certificate should be valid.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/certificates",
            body=await async_maybe_transform(
                {
                    "csr": csr,
                    "hostnames": hostnames,
                    "request_type": request_type,
                    "requested_validity": requested_validity,
                },
                origin_ca_certificate_create_params.OriginCACertificateCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OriginCACertificate]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OriginCACertificate]], ResultWrapper[OriginCACertificate]),
        )

    def list(
        self,
        *,
        zone_id: str,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[OriginCACertificate, AsyncV4PagePaginationArray[OriginCACertificate]]:
        """List all existing Origin CA certificates for a given zone.

        You can use an Origin
        CA Key as your User Service Key or an API token when calling this endpoint
        ([see above](#requests)).

        Args:
          zone_id: Identifier.

          limit: Limit to the number of records returned.

          offset: Offset the results

          page: Page number of paginated results.

          per_page: Number of records per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/certificates",
            page=AsyncV4PagePaginationArray[OriginCACertificate],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "zone_id": zone_id,
                        "limit": limit,
                        "offset": offset,
                        "page": page,
                        "per_page": per_page,
                    },
                    origin_ca_certificate_list_params.OriginCACertificateListParams,
                ),
            ),
            model=OriginCACertificate,
        )

    async def delete(
        self,
        certificate_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OriginCACertificateDeleteResponse]:
        """Revoke an existing Origin CA certificate by its serial number.

        You can use an
        Origin CA Key as your User Service Key or an API token when calling this
        endpoint ([see above](#requests)).

        Args:
          certificate_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not certificate_id:
            raise ValueError(f"Expected a non-empty value for `certificate_id` but received {certificate_id!r}")
        return await self._delete(
            path_template("/certificates/{certificate_id}", certificate_id=certificate_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OriginCACertificateDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[OriginCACertificateDeleteResponse]], ResultWrapper[OriginCACertificateDeleteResponse]
            ),
        )

    async def get(
        self,
        certificate_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OriginCACertificate]:
        """Get an existing Origin CA certificate by its serial number.

        You can use an
        Origin CA Key as your User Service Key or an API token when calling this
        endpoint ([see above](#requests)).

        Args:
          certificate_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not certificate_id:
            raise ValueError(f"Expected a non-empty value for `certificate_id` but received {certificate_id!r}")
        return await self._get(
            path_template("/certificates/{certificate_id}", certificate_id=certificate_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OriginCACertificate]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OriginCACertificate]], ResultWrapper[OriginCACertificate]),
        )


class OriginCACertificatesResourceWithRawResponse:
    def __init__(self, origin_ca_certificates: OriginCACertificatesResource) -> None:
        self._origin_ca_certificates = origin_ca_certificates

        self.create = to_raw_response_wrapper(
            origin_ca_certificates.create,
        )
        self.list = to_raw_response_wrapper(
            origin_ca_certificates.list,
        )
        self.delete = to_raw_response_wrapper(
            origin_ca_certificates.delete,
        )
        self.get = to_raw_response_wrapper(
            origin_ca_certificates.get,
        )


class AsyncOriginCACertificatesResourceWithRawResponse:
    def __init__(self, origin_ca_certificates: AsyncOriginCACertificatesResource) -> None:
        self._origin_ca_certificates = origin_ca_certificates

        self.create = async_to_raw_response_wrapper(
            origin_ca_certificates.create,
        )
        self.list = async_to_raw_response_wrapper(
            origin_ca_certificates.list,
        )
        self.delete = async_to_raw_response_wrapper(
            origin_ca_certificates.delete,
        )
        self.get = async_to_raw_response_wrapper(
            origin_ca_certificates.get,
        )


class OriginCACertificatesResourceWithStreamingResponse:
    def __init__(self, origin_ca_certificates: OriginCACertificatesResource) -> None:
        self._origin_ca_certificates = origin_ca_certificates

        self.create = to_streamed_response_wrapper(
            origin_ca_certificates.create,
        )
        self.list = to_streamed_response_wrapper(
            origin_ca_certificates.list,
        )
        self.delete = to_streamed_response_wrapper(
            origin_ca_certificates.delete,
        )
        self.get = to_streamed_response_wrapper(
            origin_ca_certificates.get,
        )


class AsyncOriginCACertificatesResourceWithStreamingResponse:
    def __init__(self, origin_ca_certificates: AsyncOriginCACertificatesResource) -> None:
        self._origin_ca_certificates = origin_ca_certificates

        self.create = async_to_streamed_response_wrapper(
            origin_ca_certificates.create,
        )
        self.list = async_to_streamed_response_wrapper(
            origin_ca_certificates.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            origin_ca_certificates.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            origin_ca_certificates.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["BundleMethod"]

BundleMethod: TypeAlias = Literal["ubiquitous", "optimal", "force"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["WorkerMetadataParam"]


class WorkerMetadataParam(TypedDict, total=False):
    """JSON-encoded metadata about the uploaded parts and Worker configuration."""

    body_part: str
    """Name of the part in the multipart request that contains the script (e.g.

    the file adding a listener to the `fetch` event). Indicates a
    `service worker syntax` Worker.
    """

    main_module: str
    """Name of the part in the multipart request that contains the main module (e.g.

    the file exporting a `fetch` handler). Indicates a `module syntax` Worker.
    """
